import { isImageFile } from '../../utils/clipboard';
import { renderUploadWidget } from './FilesUploadWidget';
import { IMG_MAX_HEIGHT } from './const';
import { getImageDimensions } from './utils';
const SUCCESS_UPLOAD_REMOVE_TIMEOUT = 1000; // 1 sec
export class CMFilesUploadPresenter {
    constructor(params) {
        this._uploadFiles = [];
        this.__reactWidget = null;
        this._cmWidget = null;
        this._state = 0 /* State.Initial */;
        this._params = params;
        this._reactContainer = document.createElement('div');
        this._reactContainer.classList.add('cm-widget-container');
        this._initialPosition = this._params.cm.getCursor();
    }
    get _reactWidget() {
        if (this.__reactWidget)
            return this.__reactWidget;
        this.__reactWidget = this._params.reactRenderer.createItem('cm-upload-files', renderUploadWidget(this._reactContainer, {
            files: this._uploadFiles,
            onReUploadClick: this._onReUploadClick.bind(this),
            onCloseClick: this._onCloseClick.bind(this),
        }));
        return this.__reactWidget;
    }
    run(files) {
        if (this._state !== 0 /* State.Initial */)
            return this;
        this._state = 1 /* State.Running */;
        for (const file of Array.from(files)) {
            this._uploadFiles.push({
                file,
                fileType: isImageFile(file) ? 'image' : 'file',
                fileName: file.name,
                status: 'uploading',
            });
            this._params.uploadHandler(file).then((res) => this._onFileUpload(Object.assign(Object.assign({}, res), { file })), (err) => this._onFileFail({ file, err }));
        }
        this._cmWidget = this._params.cm.addLineWidget(this._initialPosition.line, this._reactContainer, {});
        this._reactWidget.rerender();
        this._scheduleWidgetChange();
        return this;
    }
    destroy() {
        var _a, _b;
        this._state = 2 /* State.Destroyed */;
        (_a = this.__reactWidget) === null || _a === void 0 ? void 0 : _a.remove();
        this.__reactWidget = null;
        (_b = this._cmWidget) === null || _b === void 0 ? void 0 : _b.clear();
        this._cmWidget = null;
    }
    _onFileUpload(res) {
        if (this._state === 2 /* State.Destroyed */)
            return;
        const file = this._uploadFiles.find((item) => item.file === res.file);
        if (file) {
            file.status = 'success';
            this._reactWidget.rerender();
        }
        setTimeout(() => {
            this._insertFileMarkup(res);
            this._removeFileFromView(res.file);
        }, SUCCESS_UPLOAD_REMOVE_TIMEOUT);
    }
    _onFileFail(res) {
        if (this._state === 2 /* State.Destroyed */)
            return;
        const file = this._uploadFiles.find((item) => item.file === res.file);
        if (file) {
            file.status = 'error';
            file.errorText = String(res.err);
            this._reactWidget.rerender();
        }
    }
    _onReUploadClick(uFile) {
        if (uFile.status !== 'error')
            return;
        const file = this._uploadFiles.find((f) => f === uFile);
        if (file) {
            this._params.uploadHandler(file.file).then((res) => this._onFileUpload(Object.assign(Object.assign({}, res), { file: file.file })), (err) => this._onFileFail({ file: file.file, err }));
            file.status = 'uploading';
            file.errorText = undefined;
            this._reactWidget.rerender();
        }
    }
    _onCloseClick() {
        this._params.onAllSettled(this);
    }
    async _insertFileMarkup(res) {
        var _a, _b;
        const fileName = (_b = (_a = res.name) !== null && _a !== void 0 ? _a : res.file.name) !== null && _b !== void 0 ? _b : '';
        let markup;
        if (isImageFile(res.file)) {
            if (this._params.needDimmensionsForImages) {
                try {
                    let { height } = await getImageDimensions(res.file);
                    height = Math.min(height, IMG_MAX_HEIGHT);
                    markup = `![${fileName}](${res.url} =x${height})`;
                }
                catch (err) {
                    markup = `![${fileName}](${res.url})`;
                }
            }
            else {
                markup = `![${fileName}](${res.url})`;
            }
        }
        else {
            markup = `{% file src="${res.url}" name="${fileName.replace('"', '')}" %}`;
        }
        const isInitPosition = this._isInitialCursorPos(this._params.cm.getCursor());
        this._params.cm.replaceRange(isInitPosition ? markup + ' ' : ' ' + markup, isInitPosition
            ? this._initialPosition
            : { line: this._initialPosition.line, ch: Number.MAX_SAFE_INTEGER });
        if (isInitPosition) {
            this._initialPosition.ch += markup.length + 1;
            this._params.cm.setCursor(this._initialPosition);
        }
    }
    _removeFileFromView(file) {
        if (this._state === 2 /* State.Destroyed */)
            return;
        const index = this._uploadFiles.findIndex((item) => item.file === file);
        if (index >= 0) {
            this._uploadFiles.splice(index, 1);
            this._reactWidget.rerender();
            this._scheduleWidgetChange();
            this._checkIfAllFilesUploaded();
        }
    }
    _checkIfAllFilesUploaded() {
        if (this._state === 2 /* State.Destroyed */)
            return;
        if (!this._uploadFiles.length) {
            this._params.onAllSettled(this);
        }
    }
    _isInitialCursorPos(currentPosition) {
        return (this._initialPosition.ch === currentPosition.ch &&
            this._initialPosition.line === currentPosition.line);
    }
    _scheduleWidgetChange() {
        setTimeout(() => {
            var _a;
            (_a = this._cmWidget) === null || _a === void 0 ? void 0 : _a.changed();
        }, 20);
    }
}
